#include "Communication.h"

/**
 * Konstruktor Communication() moze vytvorit soket, ktory umozni komunikaciu po sieti prostrednictvom protokolu TCP alebo UDP.
 * @param protocol - Protokol, cez ktory je mozne komunikovat.
 * Ak <i>protocol</i> je:
 * <ul>
 * 	<li> 6, tak sa vytvori soket, ktory umozni komunikaciu po sieti prostrednictvom protokolu TCP.</li>
 * 	<li> 17, tak sa vytvori soket, ktory umozni komunikaciu po sieti prostrednictvom protokolu UDP.</li>
 * </ul>
 */

Communication::Communication(unsigned char protocol)
{
	tcpSocket = 0;
	udpSocket = 0;
	switch (protocol)
	{
		case 6:
			tcpSocket = new TCPSocket();
			break;
		case 17:
			udpSocket = new UDPSocket();
			break;
	}
}

/**
 * Ak bol soket vytvoreny, tak metoda setServer() pripoji / nastavi soketu server.
 * @param port - Port v "network byte order", na ktorom je mozne komunikovat so serverom.
 * @param address - Adresa servera v "network byte order".
 * @return Ak bol soket vytvoreny a pripojil / nastavil sa mu server, tak true inak false.
 */

bool Communication::setServer(unsigned short port, unsigned long address)
{
	if (tcpSocket != 0)
	{
		return tcpSocket->connect(port, address);
	}
	if (udpSocket != 0)
	{
		return udpSocket->setServer(port, address);
	}
	return false;
}

/**
 * Ak bol soket vytvoreny a dlzka posielanej spravy je vacsia ako 0, tak metoda send() posle cez soket data, ktore pozostavaju z dlzky posielanej spravy v "network byte order" a zo samotnej posielanej spravy bez ukoncovacieho znaku.
 * @param sendingMessage - Posielana sprava, ktora predpoklada pred svojim zaciatkom 4 volne byty.
 * @return Ak bol soket vytvoreny a dlzka posielanej spravy je rovna 0 alebo sa cez soket poslali data, tak true inak false.
 */

bool Communication::send(char *sendingMessage)
{
	if (tcpSocket != 0)
	{
		return tcpSocket->send(sendingMessage);
	}
	if (udpSocket != 0)
	{
		return udpSocket->send(sendingMessage);
	}
	return false;
}

/**
 * Ak bol soket vytvoreny, tak pokial na sokete nie su prijimane data, tak metoda select() caka, kym prijimane data nepridu alebo nevyprsi casovy interval.
 * @param seconds - Sekundy casoveho intervalu.
 * @param microseconds - Mikrosekundy casoveho intervalu.
 * @return Ak bol soket vytvoreny a do vyprsania casoveho intervalu su na sokete prijimane data, tak metoda okamzite vrati true inak false.
 */

bool Communication::select(int seconds, int microseconds)
{
	if (tcpSocket != 0)
	{
		return tcpSocket->select(seconds, microseconds);
	}
	if (udpSocket != 0)
	{
		return udpSocket->select(seconds, microseconds);
	}
	return false;
}

/**
 * Ak bol soket vytvoreny a maximalna dlzka prijimanej spravy je vacsia ako dlzka prijimanej spravy, tak metoda receive() prijme cez soket data, ktore pozostavaju z dlzky prijimanej spravy v "network byte order" a zo samotnej prijimanej spravy bez ukoncovacieho znaku.
 * @param receivingMessage - Vystupna prijimana sprava, ktora predpoklada pred svojim zaciatkom 4 volne byty.
 * @param receivingMessageSize - Maximalna dlzka prijimanej spravy.
 * @return Ak bol soket vytvoreny a maximalna dlzka prijimanej spravy je vacsia ako dlzka prijimanej spravy a cez soket sa prijali data s dlzkou vacsou ako 0, tak true inak false.
 */

bool Communication::receive(char *receivingMessage, int receivingMessageSize)
{
	if (tcpSocket != 0)
	{
		return tcpSocket->receive(receivingMessage, receivingMessageSize);
	}
	if (udpSocket != 0)
	{
		return udpSocket->receive(receivingMessage, receivingMessageSize);
	}
	return false;
}

/**
 * Ak bol soket vytvoreny, tak destruktor ~Communication() ho zatvori.
 */

Communication::~Communication()
{
	if (tcpSocket != 0)
	{
		delete tcpSocket;
	}
	else if (udpSocket != 0)
	{
		delete udpSocket;
	}
}
